@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: sock.c,v 1.46 2008/05/20 11:44:34 evertonm Exp $ */
+/* $Id: sock.c,v 1.47 2008/05/27 13:32:25 evertonm Exp $ */
 
 #include <assert.h>
 #include <netdb.h>
@@ -67,6 +67,13 @@ struct ip_mreqn {
 };
 #endif
 
+#ifndef HAVE_IP_MREQ
+struct ip_mreq {
+  struct in_addr imr_multiaddr; /* group to join */
+  struct in_addr imr_interface; /* interface to join on */
+}  
+#endif
+
 #ifndef HAVE_IPV6_MREQ
 struct ipv6_mreq {
   struct in6_addr ipv6mr_multiaddr;
@@ -414,13 +421,18 @@ static int socket_mcast_join(int sd, int family,
   switch (family) {
   case PF_INET:
     {
+#ifdef HAVE_IP_MREQN
       struct ip_mreqn opt;
-
       opt.imr_multiaddr = sa->inet.sin_addr;
       opt.imr_address.s_addr = htons(INADDR_ANY);
       opt.imr_ifindex = 0;
-
       return setsockopt(sd, SOL_IP, IP_ADD_MEMBERSHIP, &opt, sizeof(opt));
+#else
+      struct ip_mreq opt;
+      opt.imr_multiaddr = sa->inet.sin_addr;
+      opt.imr_interface.s_addr = htons(INADDR_ANY);
+      return setsockopt(sd, IPPROTO_IP, IP_ADD_MEMBERSHIP, &opt, sizeof(opt));
+#endif
     }
     break;
   case PF_INET6:
@@ -523,7 +535,10 @@ int nepim_create_socket(FILE *err,
 
   if (mcast_join) {
     assert(type == SOCK_DGRAM);
-    assert(protocol == IPPROTO_UDP);
+
+    /* Linux 2.4 getaddrinfo yields IPPROTO_UDP */
+    /* Solaris 10 getaddrinfo yields IPPROTO_IP */
+    assert(protocol == IPPROTO_IP || protocol == IPPROTO_UDP);
 
     /* SSM join (S,G) ? */
     if (ssm_src_addr) {
